// 
// AVLayerVideoGravity.cs: Implements strongly typed access for AVLayerVideoGravity
//
// Authors: Marek Safar (marek.safar@gmail.com)
//     
// Copyright 2012, Xamarin Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;

using MonoMac.Foundation;
using MonoMac.CoreFoundation;
using MonoMac.ObjCRuntime;

namespace MonoMac.AVFoundation {
#if !MONOMAC
	public enum AVLayerVideoGravity
	{
		ResizeAspect ,
		ResizeAspectFill,
		Resize
	}

	partial class AVPlayerLayer
	{
		// Should be VideoGravity only but previous binding was wrong
		public AVLayerVideoGravity LayerVideoGravity {
			set {
				NSString v;
				switch (value) {
				case AVLayerVideoGravity.ResizeAspect:
					v = AVPlayerLayer.GravityResizeAspect;
					break;
				case AVLayerVideoGravity.ResizeAspectFill:
					v = AVPlayerLayer.GravityResizeAspectFill;
					break;
				case AVLayerVideoGravity.Resize:
					v = AVPlayerLayer.GravityResize;
					break;
				default:
					throw new ArgumentOutOfRangeException ("value");
				}

				WeakVideoGravity = v;
			}
			get {
				var weak = WeakVideoGravity;
				if (weak == AVPlayerLayer.GravityResize)
					return AVLayerVideoGravity.Resize;
				if (weak == AVPlayerLayer.GravityResizeAspectFill)
					return AVLayerVideoGravity.ResizeAspectFill;

				return AVLayerVideoGravity.ResizeAspect;
			}
		}
	}

	partial class AVCaptureVideoPreviewLayer
	{
		// Should be VideoGravity only but previous binding was wrong
		public AVLayerVideoGravity LayerVideoGravity {
			set {
				NSString v;
				switch (value) {
				case AVLayerVideoGravity.ResizeAspect:
					v = AVPlayerLayer.GravityResizeAspect;
					break;
				case AVLayerVideoGravity.ResizeAspectFill:
					v = AVPlayerLayer.GravityResizeAspectFill;
					break;
				case AVLayerVideoGravity.Resize:
					v = AVPlayerLayer.GravityResize;
					break;
				default:
					throw new ArgumentOutOfRangeException ("value");
				}

				WeakVideoGravity = v;
			}
			get {
				var weak = WeakVideoGravity;
				if (weak == AVPlayerLayer.GravityResize)
					return AVLayerVideoGravity.Resize;
				if (weak == AVPlayerLayer.GravityResizeAspectFill)
					return AVLayerVideoGravity.ResizeAspectFill;

				return AVLayerVideoGravity.ResizeAspect;
			}
		}
	}

	partial class AVPlayer
	{
		public AVLayerVideoGravity? ExternalPlaybackVideoGravity {
			set {
				NSString v;
				switch (value) {
				case AVLayerVideoGravity.ResizeAspect:
					v = AVPlayerLayer.GravityResizeAspect;
					break;
				case AVLayerVideoGravity.ResizeAspectFill:
					v = AVPlayerLayer.GravityResizeAspectFill;
					break;
				case AVLayerVideoGravity.Resize:
					v = AVPlayerLayer.GravityResize;
					break;
				case null:
					v = null;
					break;
				default:
					throw new ArgumentOutOfRangeException ("value");
				}

				WeakExternalPlaybackVideoGravity = v;
			}
			get {
				var weak = WeakExternalPlaybackVideoGravity;
				if (weak == AVPlayerLayer.GravityResizeAspect)
					return AVLayerVideoGravity.ResizeAspect;
				if (weak == AVPlayerLayer.GravityResize)
					return AVLayerVideoGravity.Resize;
				if (weak == AVPlayerLayer.GravityResizeAspectFill)
					return AVLayerVideoGravity.ResizeAspectFill;

				return null;
			}
		}
	}
#endif
}
